source('genData.r')

## calculate B_li for each i, B_li is same for each l for simplicity
dg=5
basis <- bs(seq(1-5,nTime+5), df=dg)
basisPred <- NULL
for (i in 1:n) {
  basisPred[[i]] = matrix(0, nrow=dg, ncol=ni[i])
  flag = 1
  for (t in indTime) {
    if (!is.na(yData[i, t])) {
      basisPred[[i]][,flag] = predict(basis, newx=t)
      flag = flag + 1
    }
  }
}

## drop "NA" in "xData" and "yData", index the data with subject identity "i" instead of time "t"
xFinal = NULL
yFinal = NULL
Wobs = NULL
#Wsub = NULL
for (i in 1:n) {
  xFinal[[i]] = matrix(0, nrow=p, ncol=ni[i])
  yFinal[[i]] = rep(0, ni[i])
  Wobs[[i]] = diag(1, ni[i])
#  Wsub[[i]] = diag(1/ni[i], ni[i])
  indRow = 1
  for (t in 1:nTime) {
    if (!is.na(yData[i,t])) {
      xFinal[[i]][,indRow] = xData[[t]][i,]
      indRow = indRow + 1
    }
  }
  yFinal[[i]] = setdiff(yData[i,], NA)
}

## calculate gamma for each variable
gamma.obs = matrix(0, nrow=p, ncol=dg)
for (l in 1:p) {
    sum1obs = matrix(0, nrow=dg, ncol=1)
    sum2obs = matrix(0, nrow=dg, ncol=dg)
    for (i in 1:n) {
        sum1obs = sum1obs + (basisPred[[i]]%*%diag(xFinal[[i]][l,], length(xFinal[[i]][l,]))%*%Wobs[[i]]%*%yFinal[[i]])
        sum2obs = sum2obs + (basisPred[[i]]%*%diag(xFinal[[i]][l,], length(xFinal[[i]][l,]))%*%Wobs[[i]]%*%
                  diag(xFinal[[i]][l,], length(xFinal[[i]][l,]))%*%t(basisPred[[i]]))
    }
    sum2obs.inv = solve(sum2obs)
    value.obs = sum2obs.inv %*% sum1obs
    gamma.obs[l,] = value.obs
}

## select "topNumber" variables
bs <- betaScore(gamma.obs)
indSl <- selection(bs, n, topNumber=floor(2*n / (3*log(n))))
xSl <- NULL
ySl <- yFinal
for (i in 1:n) {
  xSl[[i]] = xFinal[[i]][indSl, ]
}
gammaSl <- gamma.obs[indSl,]

## group SCAD in step 1 of IVIS
print("SCAD # 1 ...")
gS <- gSCAD(xSl, ySl, gammaSl, basisPred, n, ni)

M0 = NULL
M = gS$M1
M1 = indSl[gS$M1]
print(M1)
uGamma = gS$u.gamma

####  gamma matrix for VIS algorithm - no iterative
M.screen = indSl[gS$M1]
gamma.screen = matrix(0, nrow=p, ncol=dg)
for (i in 1:length(indSl)) gamma.screen[indSl[i],] = gS$gamma[i,] #full gamma


## IVIS step 2 & 3
ite = 1
tag = 0
M.rd = NULL
M.rd[[1]] = M1
while (!setequal(M0, M1) & ite <=50 & !tag) {
  print(paste("SCAD # ",ite+1), sep="")
  lSet <- setdiff(1:p, M1)
  M0 = M1
  p.cond = length(lSet)
  
  gamma.cond = matrix(0, nrow=p.cond, ncol=dg)
  for (l in 1:p.cond) {
    sum1 = matrix(0, nrow=dg, ncol=1)
    sum2 = matrix(0, nrow=dg, ncol=dg)
    for (i in 1:n) {
      y.cond = yFinal[[i]] - uGamma[[i]]
      sum1 = sum1 + (basisPred[[i]]%*%diag(xFinal[[i]][lSet[l],], length(xFinal[[i]][lSet[l],]))%*%Wobs[[i]]%*%y.cond)
      sum2 = sum2 + (basisPred[[i]]%*%diag(xFinal[[i]][lSet[l],], length(xFinal[[i]][lSet[l],]))%*%Wobs[[i]]
             %*%diag(xFinal[[i]][lSet[l],], length(xFinal[[i]][lSet[l],]))%*%t(basisPred[[i]]))
    }
    sum2.inv = solve(sum2)
    value = sum2.inv %*% sum1
    gamma.cond[l,] = value
  }
  
  bs <- betaScore(gamma.cond)
  sl <- selection(bs, n)  
  indSl <- sort(c(lSet[sl], M0))
  #print(lSet[sl])
  #print(paste("indSl=",indSl, sep=""))
  slPos <- sum(lSet[sl]>M0)
  xSl <- NULL
  for (i in 1:n) {
    xSl[[i]] = xFinal[[i]][indSl, ]
  }
  ySl <- yFinal
  gammaSl <- gamma.obs[indSl,]
 
  gS <- gSCAD(xSl, ySl, gammaSl, basisPred, n, ni)
  M = gS$M1
  M1 = indSl[gS$M1]
  print(M1)
  uGamma = gS$u.gamma
  
  for (i in 1:ite) {
    tag = (setequal(M1, M.rd[[i]]) & ite+1-i>1)
    if (tag) {
      start.cir = i
      end.cir = ite
      break
    }
  }
  ite = ite + 1
  M.rd[[ite]] = M1
}

if (ite > 50) 	print("Warning: not converge!")

Mintersection = M1
Munion = M1
if (tag) {
  for (i in start.cir:end.cir){
    Mintersection = intersect(Mintersection, M.rd[[i]])
    Munion = union(Munion, M.rd[[i]])
  }
}

####  gamma matrix for IVIS algorithm
gamma.ivis = matrix(0, nrow=p, ncol=dg)
for (i in 1:length(indSl)) gamma.ivis[indSl[i],] = gS$gamma[i,] #full gamma

####  gamma matrix for oracle and direct fitting from IVIS selection
trueInd = seq(1:6)
xSl <- NULL
ySl <- yFinal
for (i in 1:n) {
  xSl[[i]] = xFinal[[i]][trueInd, ]
}
gamma.oracle = matrix(0, nrow=p, ncol=dg)
gamma.oracle[trueInd, ] = direct(xSl, ySl, basisPred, n, ni)

gamma.selectFit = matrix(0, nrow=p, ncol=dg)
if (setequal(M1, trueInd)) {
  gamma.selectFit = gamma.oracle
} else {
  xSl <- NULL
  for (i in 1:n) {
    xSl[[i]] = xFinal[[i]][M1, ]
    }
  gamma.selectFit[M1, ] = direct(xSl, ySl, basisPred, n, ni)
}
