library(splines)
library(mvtnorm)
n = 200
p = 500
nTime = 30
indTime = 1:30
source('trueBeta.r')
load("FullData.RData")
basis <- bs(seq(-4, 35), df=5)

tmax = 30
newT = 29/(tmax-1)*(1:tmax-1)+1
basisPred = predict(basis, newx=newT)

beta.ivis = NULL
beta.oracle = NULL
beta.screen = NULL
for (i in 1:100) {
  beta.ivis[[i]] = gamma.ivis[[i]][,] %*% t(basisPred)
  beta.oracle[[i]] = gamma.oracle[[i]][,] %*% t(basisPred)
  beta.screen[[i]] = gamma.screen[[i]][,] %*% t(basisPred)
}


tstE.ivis = rep(0, 100)
tstE.oracle = rep(0, 100)
tstE.screen = rep(0, 100)
set.seed(1234567890)

for (FLAG in 1:100) {
    source('genData.r')
    xFinal = NULL
    yFinal = NULL
    for (i in 1:n) {
      xFinal[[i]] = matrix(0, nrow=p, ncol=ni[i])
      yFinal[[i]] = rep(0, ni[i])

      indRow = 1
      for (t in 1:nTime) {
        if (!is.na(yData[i,t])) {
          xFinal[[i]][,indRow] = xData[[t]][i,]
          indRow = indRow + 1
        }
      }
      yFinal[[i]] = setdiff(yData[i,], NA)
    }
    
    ## we have beta.ivis[[Rep]]: p * nTime, with Rep = 1, 2, ..., 200
    ## xFinal[[i]]: p * ni[i] with i = 1, 2, ..., 200 for some "Rep",
    ## yFinal[[i]]: ni[i]*1 with i = 1, 2, ..., 200 for some "Rep"
    ## record.rd[[Rep]]: n * nTime
    II = seq(1,30)
    s.ivis = 0
    s.oracle = 0
    s.screen = 0
    for (i in 1:n) {
      rd = II[record[i,]==1]
      ba.ivis = beta.ivis[[FLAG]][,rd]
      ba.oracle = beta.oracle[[FLAG]][,rd]
      ba.screen = beta.screen[[FLAG]][,rd]
      yPred.ivis = apply(xFinal[[i]]*ba.ivis, 2, sum)
      yPred.oracle = apply(xFinal[[i]]*ba.oracle, 2, sum)
      yPred.screen = apply(xFinal[[i]]*ba.screen, 2, sum)
      s.ivis = s.ivis + sum((yPred.ivis-yFinal[[i]])^2)
      s.oracle = s.oracle + sum((yPred.oracle-yFinal[[i]])^2)
      s.screen = s.screen + sum((yPred.screen-yFinal[[i]])^2)
    }
    tstE.ivis[FLAG] = s.ivis / sum(record)
    tstE.oracle[FLAG] = s.oracle / sum(record)
    tstE.screen[FLAG] = s.screen / sum(record)
    FLAG = FLAG + 1  
}

save(file="tstError.RData", tstE.ivis, tstE.oracle, tstE.screen)
